=== modified file 'libdrizzle/constants.h'
--- libdrizzle/constants.h	2010-10-23 10:43:00 +0000
+++ libdrizzle/constants.h	2010-11-19 17:02:15 +0000
@@ -55,6 +55,7 @@
 #define DRIZZLE_MAX_BUFFER_SIZE          32768
 #define DRIZZLE_BUFFER_COPY_THRESHOLD    8192
 #define DRIZZLE_MAX_SERVER_VERSION_SIZE  32
+#define DRIZZLE_MAX_SERVER_EXTRA_SIZE    32
 #define DRIZZLE_MAX_SCRAMBLE_SIZE        20
 #define DRIZZLE_STATE_STACK_SIZE         8
 #define DRIZZLE_ROW_GROW_SIZE            8192

=== modified file 'libdrizzle/handshake.c'
--- libdrizzle/handshake.c	2010-09-28 07:47:48 +0000
+++ libdrizzle/handshake.c	2010-11-21 16:11:28 +0000
@@ -74,6 +74,8 @@
 drizzle_return_t drizzle_state_handshake_server_read(drizzle_con_st *con)
 {
   uint8_t *ptr;
+  int extra_length;
+  unsigned char* packet_end;
 
   drizzle_log_debug(con->drizzle, "drizzle_state_handshake_server_read");
 
@@ -91,6 +93,7 @@
     return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
   }
 
+  packet_end= con->buffer_ptr + con->packet_size;
   con->protocol_version= con->buffer_ptr[0];
   con->buffer_ptr++;
 
@@ -121,7 +124,7 @@
     return DRIZZLE_RETURN_BAD_HANDSHAKE_PACKET;
   }
 
-  if (con->packet_size != (46 + (size_t)(ptr - con->buffer_ptr)))
+  if (con->packet_size < (46 + (size_t)(ptr - con->buffer_ptr)))
   {
     drizzle_set_error(con->drizzle, "drizzle_state_handshake_server_read",
                       "bad packet size:%zu:%zu",
@@ -166,6 +169,14 @@
   memcpy(con->scramble + 8, con->buffer_ptr, 12);
   con->buffer_ptr+= 13;
 
+  /* MySQL 5.5 adds "mysql_native_password" after the server greeting. */
+  extra_length= packet_end - con->buffer_ptr;
+  assert(extra_length >= 0);
+  if (extra_length > DRIZZLE_MAX_SERVER_EXTRA_SIZE - 1)
+    extra_length= DRIZZLE_MAX_SERVER_EXTRA_SIZE - 1;
+  memcpy(con->server_extra, (char *)con->buffer_ptr, extra_length);
+  con->server_extra[extra_length]= 0;
+
   con->buffer_size-= con->packet_size;
   if (con->buffer_size != 0)
   {

=== modified file 'libdrizzle/structs.h'
--- libdrizzle/structs.h	2010-09-28 07:47:48 +0000
+++ libdrizzle/structs.h	2010-11-19 17:02:15 +0000
@@ -124,6 +124,7 @@
   char password[DRIZZLE_MAX_PASSWORD_SIZE];
   uint8_t scramble_buffer[DRIZZLE_MAX_SCRAMBLE_SIZE];
   char server_version[DRIZZLE_MAX_SERVER_VERSION_SIZE];
+  char server_extra[DRIZZLE_MAX_SERVER_EXTRA_SIZE];
   drizzle_state_fn *state_stack[DRIZZLE_STATE_STACK_SIZE];
   char user[DRIZZLE_MAX_USER_SIZE];
 };

